import java.awt.*;
import java.awt.event.*;
import java.util.*;
import GenList;

public synchronized class ButtonList extends Panel implements GenList {
  public interface ButtonListListener {
    public void ActionText(String s);
  }

  // A panel that doesn't get wider than some other container
  static private class InnerPanel extends Panel {
    Container c;
    int hbuf;
    
    public InnerPanel(LayoutManager lm, Container c, int hbuf) {
      super(lm);
      this.c = c;
      this.hbuf=hbuf;
    }
    public Dimension getPreferredSize() {
      Dimension d = super.getPreferredSize();
      Rectangle cd = c.getBounds();
      int w,h,nw,nh,area;
      w = d.width;
      h = d.height;
      area = w*h;
      nw = cd.width-hbuf;
      if (nw==0) nw=1;
      nh = area/nw+cd.height; 
      return(new Dimension(nw,nh));
    }
    public Dimension getMinimumSize() {
      return(getPreferredSize());
    }
  }

  static private class ButtonProxy implements ActionListener {
    ButtonListListener b;
    String s;

    public ButtonProxy(ButtonListListener b, String s) {
      this.s = s;
      this.b = b;
    }
    public void actionPerformed(ActionEvent e) {
      b.ActionText(s);
    }
  }

  CardLayout layout;
  ScrollPane scrollpane;
  InnerPanel innerpanel;
  //  Vector buttonVector;
  int rows;
  ButtonListListener action;

  void startup() {
    layout = new CardLayout();
    setLayout(layout);
    
    scrollpane = new ScrollPane();
    innerpanel = new InnerPanel(new FlowLayout(FlowLayout.LEFT,0,0),
				this, scrollpane.getVScrollbarWidth()+30);
    scrollpane.add(innerpanel);
    add(scrollpane,"ButtonList");
    layout.show(this,"ButtonList");
    
    //    buttonVector = new Vector();
  }

  public void removeAll() {
    super.removeAll();
    startup();
    doLayout();
  }

  public ButtonList(int rows) { // rows is just for sizing
    action=null;
    this.rows=rows;
    startup();
  }

  public ButtonList(int rows, ButtonListListener action) {
    this.action=action;
    this.rows=rows;
    startup();
  }
  
  public void addLabel(String s) {
    Label l = new Label(s,Label.CENTER);
    innerpanel.add(l);
    //    l.setFont(getParent().getFont());
    validateTree();
  }

  public void addLabel(String s, Color fg, Color bg) {
    Label l = new Label(s,Label.CENTER);
    l.setForeground(fg);
    l.setBackground(bg);
    innerpanel.add(l);
    //    l.setFont(getParent().getFont());
    validateTree();
  }

  public void add(String s) {
    add(s,getForeground(),getBackground());
  }

  public void add(String s, Color fg, Color bg) {
    Button b;
    if (s.length()==0)
      return;
    else {
      char c;
      c = s.charAt(0);
      b = new Button("" + c);
      b.setForeground(fg);
      b.setBackground(bg);
      //buttonVector.addElement(new ButStr(b,s));
      innerpanel.add(b);
      if (action!=null) {
	b.addActionListener(new ButtonProxy(action,s));
      }
    }
    validateTree();    
  }
}

